<?php

namespace App\Http\Controllers\V1\Canje;

use App\Enums\EstatusCanje;
use App\Http\Controllers\Controller;
use App\Models\NivelReto;
use App\Models\Recompensa;
use App\Models\Canje;
use App\Models\Participante;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Mail;

class CanjeController extends Controller
{

    public function getRecompensasCanje()
    {
        try {
            $canjes = Canje::join('participantes', 'canjes.participante_id', "=", "participantes.id")
                ->with("recompensa.files")
                ->join('recompensas', 'canjes.recompensa_id', "=", "recompensas.id")
                ->join('files', 'recompensas.file_id', "=", "files.id")
                ->select(
                    "canjes.id",
                    "canjes.proceso_estatus",
                    "canjes.puntos_canjeados",
                    "canjes.fecha_canje",
                    "canjes.numero_canje",
                    "canjes.created_at",
                    "participantes.nombre_completo",
                    "recompensas.id as recompensa_id",
                    "recompensas.file_id",
                    "recompensas.nombre as nombre_recompensa",
                    "recompensas.descripcion_premio",
                    "recompensas.meta_canje"
                )
                ->orderBy("canjes.numero_canje", "asc")
                ->get();
            return response()->json($canjes);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los canjes"]]], 400);
        }
    }

    public function getRecompensasCanjeByParticipante($idParticipante)
    {
        try {
            $canjes = Canje::join('participantes', 'canjes.participante_id', "=", "participantes.id")
                ->with("recompensa.files")
                ->where("canjes.participante_id", $idParticipante)
                ->join('recompensas', 'canjes.recompensa_id', "=", "recompensas.id")
                ->join('files', 'recompensas.file_id', "=", "files.id")
                ->select(
                    "canjes.id",
                    "canjes.proceso_estatus",
                    "canjes.puntos_canjeados",
                    "canjes.fecha_canje",
                    "canjes.numero_canje",
                    "canjes.created_at",
                    "canjes.recompensa_id",
                    "participantes.nombre_completo",
                    "recompensas.id as recompensa_id",
                    "recompensas.file_id",
                    "recompensas.nombre as nombre_recompensa",
                    "recompensas.descripcion_premio",
                    "recompensas.meta_canje"
                )
                ->orderBy("canjes.numero_canje", "asc")
                ->get();
            return response()->json($canjes);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus canjes"]]], 400);
        }
    }

    public function getPuntosCanjeParticipante($idParticipante)
    {
        try {

            $totalPuntosAcumulados = NivelReto::whereHas('recompensa', function ($query) use ($idParticipante) {
                $query->where('historial_recompensas.participante_id', $idParticipante);
            })->get()->pluck('puntos')->sum();
            $totalPuntosCanjeados = Canje::where("participante_id", $idParticipante)->get()->pluck("puntos_canjeados")->sum();
            $totalPuntosParticipanteTotales = $totalPuntosAcumulados - $totalPuntosCanjeados;
            return response()->json(["puntos" => $totalPuntosParticipanteTotales]);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus puntos de canje"]]], 400);
        }
    }

    public function createCanjeParticipante(Request $request)
    {
        //Validamos los datos
        $data = $request->only('participante_id', 'recompensa_id');
        $validator = Validator::make($data, [
            'participante_id' => 'integer|required|exists:participantes,id',
            'recompensa_id' => 'integer|required|exists:recompensas,id',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $idParticipante = $request->participante_id;

            $recompensaCanje = Recompensa::find($request->recompensa_id);
            $puntosCaje = $recompensaCanje->meta_canje;
            $totalPuntosAcumulados = NivelReto::whereHas('recompensa', function ($query) use ($idParticipante) {
                $query->where('historial_recompensas.participante_id', $idParticipante);
            })->get()->pluck('puntos')->sum();

            $totalPuntosCanjeados = Canje::where("participante_id", $idParticipante)->get()->pluck("puntos_canjeados")->sum();
            $totalPuntosParticipanteTotales = $totalPuntosAcumulados - $totalPuntosCanjeados;
            if (!($totalPuntosParticipanteTotales >= $puntosCaje)) {
                return response()->json(['error' => ['mensaje' => ["No tienes suficientes puntos para canjear la recompensa"]]], 400);
            }
            $newCaje = Canje::create([
                "participante_id" => $request->participante_id,
                "recompensa_id" => $request->recompensa_id,
                "puntos_canjeados" => $puntosCaje,
            ]);
            return response()->json([
                'message' => 'La recompensa se entrego con exito correctamente',
                'data' => $newCaje
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al cajear la recompensa"]]], 400);
        }
    }

    public function changeStatusCanje(Request $request)
    {
        //Validamos los datos
        $data = $request->only('status', 'canje_id');
        $validator = Validator::make($data, [
            'status' => 'string|required',
            'canje_id' => 'integer|required|exists:canjes,id',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $currentDateTime = Carbon::now();
            $dateNow = $currentDateTime->format('Y-m-d');
            $updateCanje = Canje::Find($request->canje_id);
            $updateCanje->update([
                "proceso_estatus" => $request->status,
                "fecha_canje" => $dateNow
            ]);
            $participante = Participante::with("usuario")
                ->where("id", $updateCanje->participante_id)->first();

            $recompensa = Recompensa::where("id", $updateCanje->recompensa_id)->first();

            if ($request->status == "aprobado") {
                $details = [
                    'nombre_participante' => $participante->nombre_completo,
                    'num_canje' => $updateCanje->numero_canje,
                    'nombre_recompensa' => $recompensa->nombre,
                    'num_puntos_canjeados' => $updateCanje->puntos_canjeados
                ];
                Mail::to($participante->usuario['email'])->send(new \App\Mail\NotificacionAprobadoCanje($details));
            }
            return response()->json([
                'message' => 'Se actuliazó correctamente el estatus del canje',
                'data' => $updateCanje
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el status del canje"]]], 400);
        }
    }

    public function getCanjeDetalle($idCanje)
    {
        try {
            $canjes = Canje::join('participantes', 'canjes.participante_id', "=", "participantes.id")
                ->with("recompensa.files")
                ->join('recompensas', 'canjes.recompensa_id', "=", "recompensas.id")
                ->join('files', 'recompensas.file_id', "=", "files.id")
                ->where("canjes.id", $idCanje)
                ->select(
                    "canjes.id",
                    "canjes.proceso_estatus",
                    "canjes.puntos_canjeados",
                    "canjes.fecha_canje",
                    "canjes.numero_canje",
                    "canjes.created_at",
                    "participantes.nombre_completo",
                    "recompensas.id as recompensa_id",
                    "recompensas.nombre as nombre_recompensa",
                    "recompensas.descripcion_premio",
                    "recompensas.meta_canje",
                    "recompensas.file_id"
                )
                ->orderBy("canjes.numero_canje", "asc")
                ->first();
            return response()->json($canjes);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error el detalle del canje"]]], 400);
        }
    }
}
