<?php

namespace App\Http\Controllers\V1\Evidencia;

use App\Enums\EstatusEvidencia;
use App\Http\Controllers\Controller;
use App\Models\Evidencia;
use App\Models\EvidenciaRegistro;
use App\Models\EvidenciaReto;
use App\Models\HistorialRecompensas;
use App\Models\NivelReto;
use App\Models\Reto;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;

class EvidenciaController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try {
            $evidencias = Evidencia::with(['files', 'retos', 'participante', 'registros_evidencias.producto'])->get();
            return $evidencias;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ['Error al consultar las evidencias']]], 400);
        }
    }

    /**
     * Obtiene la informacion de un producto
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $evidencia = Evidencia::with(['files', 'retos', 'participante', 'usuario', 'registros_evidencias.producto'])
                ->where("id", $id)
                ->first();
            return response()->json($evidencia);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar la evidencia"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only('comentario', 'participante_id', 'file_id', 'user_id', 'productos');
        $validator = Validator::make($data, [
            'comentario' => 'nullable|string|max:1200',
            'participante_id' => 'required|integer|exists:participantes,id',
            'user_id' => 'required|integer|exists:users,id',
            'file_id' => 'integer|required',
            'productos' => 'required|array|min:1',
            'productos.*.id' => 'required|integer|exists:productos,id',
            'productos.*.cantidad' => 'required|integer',
            'productos.*.monto' => 'required|numeric|between:0,99999.99',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $currentDateTime = Carbon::now();
            $dateNow = $currentDateTime->format('Y-m-d');
            DB::transaction(function () use ($request, $dateNow) {
                $comentario = $request->comentario;
                $participante_id = $request->participante_id;
                $evidenciaNew = Evidencia::create([
                    "fecha_captura" => $dateNow,
                    "comentario" => $comentario,
                    "user_id" => $request->user_id,
                    "participante_id" => $participante_id,
                    "file_id" => $request->file_id,
                ]);
                foreach ($request->productos as $producto) {
                    $idProducto = $producto['id'];
                    $cantidad = $producto['cantidad'];
                    $monto = $producto['monto'];
                    $precioProducto = number_format($monto / $cantidad, 2);

                    EvidenciaRegistro::create([
                        "cantidad" => $cantidad,
                        "monto" => $monto,
                        "precio_producto" => $precioProducto,
                        "producto_id" => $idProducto,
                        "evidencia_id" => $evidenciaNew->id
                    ]);
                }
                $retosQueryFilter = Reto::with(['nivel_retos' => function ($query) use ($dateNow) {
                    $query->where('nivel_retos.fecha_inicio', '<=', $dateNow);
                    $query->where('nivel_retos.fecha_fin', '>=', $dateNow);
                }])->whereHas('participantes', function ($query) use ($request) {
                    $query->where('participantes.id', $request->participante_id);
                })
                    ->whereHas('productos', function ($query) use ($idProducto) {
                        $query->where('productos.id', $idProducto);
                    })
                    ->get();
                if (!empty($retosQueryFilter)) {
                    foreach ($retosQueryFilter as $reto) {
                        $retoId = $reto->id;
                        $exitsEvidenciaReto = DB::table('evidencia_retos')
                            ->where('reto_id', '=', $retoId)
                            ->where('evidencia_id', '=', $evidenciaNew->id)->exists();
                        if (!$exitsEvidenciaReto) {
                            EvidenciaReto::create([
                                'reto_id' => $retoId,
                                'evidencia_id' => $evidenciaNew->id
                            ]);
                        }
                    }
                }
            });
            return response()->json([
                'message' => 'Las evidencias se crearon correctamente',
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al crear la evidencia"]]], 400);
        }
    }

    public function evaluarEvidencia(Request $request, $id)
    {
        $data = $request->only('estatus', 'user_id');
        $validator = Validator::make($data, [
            'estatus' => 'required|string',
            'user_id' => 'required|integer|exists:users,id',
        ]);
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $evidenciaUpdate = DB::transaction(function () use ($request, $id) {
                $evidenciaFind = Evidencia::findOrfail($id);
                $evidenciaFind->update([
                    "estatus" =>  $request->estatus,
                ]);
                if ($request->estatus !== EstatusEvidencia::APROBADO) {
                    return response()->json([
                        'message' => 'Se evalúo correctamente la evidencia'
                    ], Response::HTTP_OK);
                }

                $fechaCapturaEvidencia = $evidenciaFind->fecha_captura;
                $idParticipante = $evidenciaFind->participante_id;
                //consultos niveles que esten relacionados a la evidencia tomando en cuenta la fecha de captura en
                // la que se creo la evidencia y que no tengan una recompensa asignada, la recompensa me indica si fue cumplido
                $nivelesReto = NivelReto::with(['reto_detail'])
                    ->whereHas('reto_detail.evidencias_retos', function ($query) use ($id) {
                        $query->where('evidencia_id', $id);
                    })
                    ->whereDoesntHave('recompensa')
                    ->where('nivel_retos.fecha_inicio', '<=', $fechaCapturaEvidencia)
                    ->where('nivel_retos.fecha_fin', '>=', $fechaCapturaEvidencia)
                    ->get();
                //recorro los nivels para validar si cumplio algun nivel del reto
                foreach ($nivelesReto as $nivelReto) {
                    if ($this->cumpleNivelReto($nivelReto, $idParticipante)) {
                        HistorialRecompensas::create([
                            "participante_id" => $idParticipante,
                            "nivel_id" => $nivelReto->id,
                            "user_id" => $request->user_id,
                        ]);
                    }
                }
                return $evidenciaFind;
            });
            return response()->json([
                'message' => 'Se aprobo correctamente la evidencia',
                'data' => $evidenciaUpdate
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al aprobar la evidencia"]]], 400);
        }
    }

    private function cumpleNivelReto($nivelReto, $idParticipante)
    {
        //id del reto principanl
        $idReto = $nivelReto->reto_detail['id'];
        $meta = $nivelReto['meta'];
        $tipoReto = $nivelReto->reto_detail['tipo_reto'];

        $evidenciaQuery = EvidenciaRegistro::with(['evidencia_detail'])
            ->whereHas('evidencia_detail', function ($query) use ($idParticipante) {
                $query->where('estatus', EstatusEvidencia::APROBADO);
                $query->where('participante_id', $idParticipante);
            })
            ->whereHas('evidencia_detail.retos', function ($query) use ($idReto) {
                $query->where('reto_id', $idReto);
            });

        $totalData = $evidenciaQuery->selectRaw('SUM(cantidad) as total_cantidad, SUM(monto) as total_monto')
            ->first();

        $cantidadTotal = $totalData->total_cantidad;
        $cantidadMonto = $totalData->total_monto;

        return ($tipoReto === 'cantidad' && $cantidadTotal >= $meta) ||
            ($tipoReto === 'dinero' && $cantidadMonto >= $meta);
    }
}
