<?php

namespace App\Http\Controllers\V1\Participante;

use App\Http\Controllers\Controller;
use App\Models\Participante;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Mail;
use Spatie\Permission\Models\Role;

class ParticipanteController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try {
            $allParticipantes = Participante::with(['retos', 'files', 'usuario.roles'])->get();
            return $allParticipantes;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los participantes"]]], 400);
        }
    }

     /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getParticipanteRole($role)
    {
        try {
            $allParticipantes = Participante::with(['retos', 'files', 'usuario.roles'])
            ->whereHas('usuario.roles', function ($query) use ($role) {
                $query->where('name', $role);
            })
            ->get();
            return $allParticipantes;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los participantes"]]], 400);
        }
    }

    /**
     * Obtiene la informacion de un participante
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $queryParticipante = Participante::with(['retos', 'files', 'usuario.roles'])
            ->where('id', $id)->firstOrFail();
            return $queryParticipante;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el participante"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only(
            'email',
            'nombre_tienda',
            'direccion',
            'nombre_completo',
            'localidad',
            'estado',
            'ciudad',
            'ruta',
            'file_id',
            'role',
            'identificador'
        );
        $validator = Validator::make($data, [
            'email' => 'required|email|unique:users',
            'nombre_tienda' => 'nullable|max:400|string',
            'direccion' => 'nullable|string',
            'nombre_completo' => 'required|max:400|string',
            'localidad' => 'nullable|max:400|string',
            'estado' => 'nullable|max:400|string',
            'ciudad' => 'nullable|max:400|string',
            'file_id' => 'nullable|integer',
            'role' => 'required|string',
            'identificador' => 'required|string|unique:participantes,identificador'
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $randomPassword = Str::random(12);
            $participanteNew = DB::transaction(function () use ($request, $randomPassword) {
                $encriptPassword = bcrypt($randomPassword);
                //Creamos el nuevo usuario
                $userCreate = User::create([
                    'email' => $request->email,
                    'password' => $encriptPassword
                ]);
                if ($request->role == "repartidor" || $request->role == "tendero") {
                    //le asigno el rol con la funcion de la libreria de permisos
                    $userCreate->assignRole($request->role);
                }

                $participante = Participante::create([
                    "nombre_tienda" => $request->nombre_tienda,
                    "direccion" => $request->direccion,
                    "nombre_completo" => $request->nombre_completo,
                    "localidad" => $request->localidad,
                    "estado" => $request->estado,
                    "ciudad" => $request->ciudad,
                    "ruta" => $request->ruta,
                    "file_id" => $request->file_id,
                    "user_id" => $userCreate->id,
                    "identificador" => $request->identificador
                ]);

                return $participante;
            });

            $urlLogin = \config('app_urls.confirm_url');
            DB::afterCommit(function ()  use ($request, $urlLogin, $randomPassword) {
                //envio de correo invitacion
                $details = [
                    'nombre_completo' => $request->nombre_completo,
                    'password' => $randomPassword,
                    'url' => $urlLogin
                ];
                //se envia el correo
                Mail::to($request->email)->send(new \App\Mail\InvitacionPersona($details));
            });

            return response()->json([
                'message' => 'El participante se creó correctamente',
                'data' => $participanteNew
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al crear el participante"]]], 400);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Participante  $participante
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //Validamos los datos
        $data = $request->only(
            'nombre_tienda',
            'direccion',
            'nombre_completo',
            'localidad',
            'estado',
            'ciudad',
            'ruta',
            'file_id'
        );
        $validator = Validator::make($data, [
            'nombre_tienda' => 'nullable|max:400|string',
            'direccion' => 'nullable|string',
            'nombre_completo' => 'required|max:400|string',
            'localidad' => 'nullable|max:400|string',
            'estado' => 'nullable|max:400|string',
            'ciudad' => 'nullable|max:400|string',
            'file_id' => 'nullable|integer'
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $participanteFind = Participante::findOrfail($id);
            $participanteFind->update([
                "nombre_tienda" => $request->nombre_tienda,
                "direccion" => $request->direccion,
                "nombre_completo" => $request->nombre_completo,
                "localidad" => $request->localidad,
                "estado" => $request->estado,
                "ciudad" => $request->ciudad,
                "ruta" => $request->ruta,
                "file_id" => $request->file_id
            ]);
            return response()->json([
                'message' => 'El participante se actualizo correctamente',
                'data' => $participanteFind
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el participante"]]], 400);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Participante  $participante
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            $participanteFind = Participante::find($id);
            if (!$participanteFind)
                return response()->json(['error' => ['mensaje' => ["El participante no existe"]]], 400);

            $existRelacionesParticipante = $participanteFind->secureDelete("retos");
            if ($existRelacionesParticipante)
                return response()->json(['error' => ['mensaje' => ["La participante ha sido asignado a los retos"]]], 400);
            //Buscamos el usuario a eliminar
            $userDelete = User::findOrfail($participanteFind->user_id);
            $participanteFind->delete();
            $userDelete->delete();

            return response()->json([
                'message' => 'Se eliminó el participante correctamente'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el participante"]]], 400);
        }
    }
}
