<?php

namespace App\Http\Controllers\V1\Perfil;

use App\Http\Controllers\Controller;
use App\Models\Participante;
use App\Models\User;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Symfony\Component\HttpFoundation\Response;

class PerfilController extends Controller
{

    /**
     * The function `getPerfil` retrieves a user's profile information along with their associated
     * participant data in PHP.
     * 
     * @param userId The `getPerfil` function is designed to retrieve the profile information for a
     * user with the given `userId`. The function attempts to find the user record along with their
     * associated `participante` relationship using Eloquent ORM.
     * 
     * @return The `getPerfil` function is returning the user data along with their associated
     * `participante` data if found based on the provided ``. If an error occurs during the
     * query execution, it will return a JSON response with an error message indicating "Error al
     * consultar el perfil" and a status code of 400.
     */
    public function getPerfil($userId){
        try {
            $userQuery = User::leftJoin("participantes", "users.id", '=', "participantes.user_id")
            ->where('users.id', $userId)
            ->select("users.id", "users.name", "users.email", "participantes.nombre_completo",
            "participantes.localidad", "participantes.estado", "participantes.ciudad", "participantes.nombre_tienda",
            "participantes.direccion")
            ->first();
            return $userQuery;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => [$e->getMessage()]]], 400);
        }
    }


    /**
     * The function `updatePerfil` updates user profile information including email, name, address, and
     * store details, handling validation and error cases.
     * 
     * @param Request request The `updatePerfil` function is a PHP function that handles updating user
     * profile information based on the data provided in the request. Let's break down the function and
     * its functionality:
     * 
     * @return The function `updatePerfil` is returning a JSON response. If the validation fails, it
     * returns an error message with the validation messages. If the update process is successful, it
     * returns a success message along with the updated user and participant data in the JSON format.
     * If an exception occurs during the update process, it returns an error message indicating the
     * failure to update the profile.
     */
    public function updatePerfil(Request $request)
    {
        //Validamos los datos
        $data = $request->only(
            'email',
            'nombre_tienda',
            'direccion',
            'nombre_completo',
            'localidad',
            'estado',
            'ciudad',
            'user_id'
        );
        $validator = Validator::make($data, [
            'email' => 'nullable|email|unique:users',
            'nombre_tienda' => 'nullable|max:400|string',
            'direccion' => 'nullable|string',
            'nombre_completo' => 'required|max:400|string',
            'localidad' => 'nullable|max:400|string',
            'estado' => 'nullable|max:400|string',
            'ciudad' => 'nullable|max:400|string',
            'user_id' => 'required|integer|exists:users,id',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $participante = Participante::where("user_id", $request->user_id)->first();
            if($participante){
                $participante->update([
                    "nombre_tienda" => $request->nombre_tienda,
                    "direccion" => $request->direccion,
                    "nombre_completo" => $request->nombre_completo,
                    "localidad" => $request->localidad,
                    "estado" => $request->estado,
                    "ciudad" => $request->ciudad
                ]);
            }
            $userUpdate = User::find($request->user_id);
            $dataUpdate = [
                "name" => $request->nombre_completo,
            ];
            if($request->email){
                $dataUpdate["email"] = $request->email;
            }
            $userUpdate->update($dataUpdate);
            return response()->json([
                'message' => 'Se actuliazo correctamente el usuario',
                'data' => [
                    "email" => $userUpdate->email,
                    "localidad" => $participante->localidad ?? '', 
                    "nombre_completo" => $participante->nombre_completo ?? $userUpdate->name,
                    "estado" => $participante->estado ?? '',
                    "ciudad" => $participante->ciudad ?? '',
                    "direccion" => $participante->direccion ?? '',
                    "nombre_tienda" => $participante->nombre_tienda ?? ''
                ]
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el perfil"]]], 400);
        }
    }
}
