<?php

namespace App\Http\Controllers\V1\Producto;

use App\Http\Controllers\Controller;
use App\Models\Producto;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;

class ProductoController extends Controller
{

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try {
            $productos = Producto::with('files')->get(); 
            return $productos;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los productos"]]], 400);
        }
    }

     /**
     * Obtiene la informacion de un producto
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $producto = Producto::with('files')->where("id", $id)->first();
            return response()->json($producto);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el producto"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only('nombre', 'descripcion', 'user_id', 'file_id');
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion' => 'nullable|max:600|string',
            'user_id' => 'required|integer|exists:users,id',
            'file_id' => 'integer|required',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $producto = Producto::create([
                "nombre" => $request->nombre,
                "descripcion" => $request->descripcion,
                "user_id" => $request->user_id,
                'file_id' => $request->file_id
            ]);
            return response()->json([
                'message' => 'El producto se creó correctamente',
                'data' => $producto
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al crear el producto"]]], 400);
        }
    }

     /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Producto  $producto
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //Validamos los datos
        $data = $request->only('nombre', 'descripcion', 'estatus', 'file_id');
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion' => 'nullable|max:600|string',
            'estatus' => 'required|boolean',
            'file_id' => 'integer|required',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $productoFind = Producto::findOrfail($id);
            $productoFind->update([
                "nombre" => $request->nombre,
                "descripcion" => $request->descripcion,
                "estatus" => $request->estatus,
                'file_id' => $request->file_id
            ]);
            return response()->json([
                'message' => 'El producto se actualizo correctamente',
                'data' => $productoFind
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al actualizar el producto"]]], 400);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Producto  $producto
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            $productoDelete = Producto::find($id);
            if (!$productoDelete)
                return response()->json(['error' => ['mensaje' => ["El producto no existe"]]], 400);
            $productoDelete->delete();
            return response()->json([
                'message' => 'Se eliminó el producto correctamente'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el producto"]]], 400);
        }
    }
}
