<?php

namespace App\Http\Controllers\V1\Reto;

use App\Http\Controllers\Controller;
use App\Models\NivelReto;
use App\Models\Reto;
use App\Models\RetoParticipantes;
use App\Models\RetoProducto;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

class RetoController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        try {
            $retos = Reto::with([
                "participantes",
                "productos",
                'nivel_retos.insignias.files', 'files',
                'evidencias_retos' => function ($query) {
                    $query->orderBy('fecha_captura', 'desc');
                }
            ])
                ->get();
            return $retos;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los retos"]]], 400);
        }
    }

    public function retosByTypeParticipante($tipo_reto)
    {
        try {
            $retos = Reto::with([
                "participantes",
                "productos",
                'nivel_retos.insignias.files', 'files',
                'evidencias_retos' => function ($query) {
                    $query->orderBy('fecha_captura', 'desc');
                }
            ])
            ->where("tipo_participante", $tipo_reto)
                ->get();
            return $retos;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar los retos"]]], 400);
        }
    }

    /**
     * Obtiene la informacion de un reto
     *
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        try {
            $retoQuery = Reto::with([
                "participantes",
                "productos",
                'nivel_retos.insignias.files'  => function ($query) {
                    $query->orderBy('created_at', 'asc');
                }, 'files'
            ])
                ->where('id', $id)->first();
            return $retoQuery;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el reto"]]], 400);
        }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //Validamos los datos
        $data = $request->only('nombre', 'descripcion', 'tipo_reto', 'tipo_participante', 'user_id', 'productos', 'file_id', 'niveles');
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion' => 'nullable|max:600|string',
            'tipo_reto' => 'required|string',
            'tipo_participante' => ['required', 'max:500', 'string', Rule::in(['tendero', 'repartidor'])],
            'user_id' => 'required|integer|exists:users,id',
            'file_id' => 'integer|required',
            'productos' => 'required|array',
            'productos.*.id' => 'required|integer|exists:productos,id',
            'niveles' => 'required|array',
            'niveles.*.nombre' => "required|max:300|string",
            'niveles.*.meta' => "required|integer",
            'niveles.*.puntos' => "required|integer",
            'niveles.*.fecha_inicio' => "required|date",
            'niveles.*.fecha_fin' => "required|date|after_or_equal:fecha_inicio",
            'niveles.*.insignia_id' =>  'required|integer|exists:insignias,id',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $retoCreateNew = DB::transaction(function () use ($request) {
                $retoCreate = Reto::create([
                    "nombre" => $request->nombre,
                    "descripcion" => $request->descripcion,
                    "tipo_reto" => $request->tipo_reto,
                    "tipo_participante" => $request->tipo_participante,
                    "user_id" => $request->user_id,
                    'file_id' => $request->file_id
                ]);
                foreach ($request->productos as $valor) {
                    $idProducto = $valor['id'];
                    RetoProducto::create([
                        "reto_id" => $retoCreate->id,
                        "producto_id" => $idProducto,
                    ]);
                }
                foreach ($request->niveles as $valor) {
                    $nombre = $valor['nombre'];
                    $meta = $valor['meta'];
                    $puntos = $valor['puntos'];
                    $fechaInicio = $valor['fecha_inicio'];
                    $fechaFin = $valor['fecha_fin'];
                    $insigniaId = $valor['insignia_id'];
                    NivelReto::create([
                        "nombre" => $nombre,
                        "meta" => $meta,
                        "puntos" => $puntos,
                        "fecha_inicio" => $fechaInicio,
                        "fecha_fin" => $fechaFin,
                        "insignia_id" => $insigniaId,
                        "reto_id" => $retoCreate->id,
                    ]);
                }
                return $retoCreate;
            });
            return response()->json([
                'message' => 'El reto se creó correctamente',
                'data' => $retoCreateNew
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al crear el reto"]]], 400);
        }
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \App\Models\Reto  $reto
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //Validamos los datos
        $data = $request->only('nombre', 'descripcion', 'tipo_reto', 'productos', 'file_id', 'niveles');
        $validator = Validator::make($data, [
            'nombre' => 'required|max:400|string',
            'descripcion' => 'nullable|max:600|string',
            'tipo_reto' => 'required|string',
            'file_id' => 'integer|required',
            'productos' => 'required|array|min:1',
            'productos.*.id' => 'required|integer|exists:productos,id',
            'niveles' => 'required|array|min:1',
            'niveles.*.id' => "required",
            'niveles.*.nombre' => "required|max:300|string",
            'niveles.*.meta' => "required|integer",
            'niveles.*.puntos' => "required|integer",
            'niveles.*.fecha_inicio' => "required|date",
            'niveles.*.fecha_fin' => "required|date|after_or_equal:fecha_inicio",
            'niveles.*.insignia_id' =>  'required|integer|exists:insignias,id',
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            $retoUpdate = DB::transaction(function () use ($request, $id) {
                $retoFind = Reto::findOrfail($id);
                $retoFind->update([
                    "nombre" => $request->nombre,
                    "descripcion" => $request->descripcion,
                    "tipo_reto" => $request->tipo_reto,
                    'file_id' => $request->file_id
                ]);
                $idFilterNotExitsNiveles = array_column($request->niveles, 'id');
                $nivelesQueryDelete = NivelReto::where('reto_id', $id)
                    ->whereDoesntHave('recompensa')
                    ->whereNotIn('id', $idFilterNotExitsNiveles)
                    ->get();
                $nivelesQueryDelete = $nivelesQueryDelete->toArray();
                if (!empty($nivelesQueryDelete)) {
                    foreach ($nivelesQueryDelete as $valor) {
                        $idNivel = $valor->id;;
                        $NivelDeleteFind = NivelReto::findOrfail($idNivel);
                        $NivelDeleteFind->delete();
                    }
                }

                foreach ($request->niveles as $valor) {
                    $idNivel = $valor['id'];
                    $nombre = $valor['nombre'];
                    $meta = $valor['meta'];
                    $puntos = $valor['puntos'];
                    $fechaInicio = $valor['fecha_inicio'];
                    $fechaFin = $valor['fecha_fin'];
                    $insigniaId = $valor['insignia_id'];
                    $nivelUpdate = NivelReto::find($idNivel);
                    if ($nivelUpdate) {
                        $nivelUpdate->update([
                            "nombre" => $nombre,
                            "meta" => $meta,
                            "puntos" => $puntos,
                            "fecha_inicio" => $fechaInicio,
                            "fecha_fin" => $fechaFin,
                            "insignia_id" => $insigniaId
                        ]);
                    } else {
                        NivelReto::create([
                            "nombre" => $nombre,
                            "meta" => $meta,
                            "puntos" => $puntos,
                            "fecha_inicio" => $fechaInicio,
                            "fecha_fin" => $fechaFin,
                            "insignia_id" => $insigniaId,
                            "reto_id" => $id
                        ]);
                    }
                }

                $idFilterNotExits = array_column($request->productos, 'producto_id');
                $productRetoToDelete = DB::table('reto_productos')
                    ->where('reto_id', $id)
                    ->whereNotIn('producto_id', $idFilterNotExits)->get();
                $productRetoRelationArrayDelete = $productRetoToDelete->toArray();

                if (!empty($productRetoRelationArrayDelete)) {
                    foreach ($productRetoRelationArrayDelete as $valor) {
                        $idRetoProduct = $valor->id;
                        $retoProductQuery = RetoProducto::findOrfail($idRetoProduct);
                        $retoProductQuery->delete();
                    }
                }

                foreach ($request->productos as $valor) {
                    $idProducto = $valor['id'];
                    $exitsRetoProducto = DB::table('reto_productos')
                        ->where('reto_id', '=', $id)
                        ->where('producto_id', '=', $idProducto)->exists();
                    if (!$exitsRetoProducto) {
                        RetoProducto::create([
                            "reto_id" => $id,
                            "producto_id" => $idProducto,
                        ]);
                    }
                }
                return $retoFind;
            });

            return response()->json([
                'message' => 'El reto se actualizo correctamente',
                'data' => $retoUpdate
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => [$e->getMessage()]]], 400);
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  \App\Models\Reto  $reto
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        try {
            $retoDelete = Reto::find($id);
            if (!$retoDelete)
                return response()->json(['error' => ['mensaje' => ["El reto no existe"]]], 400);
            $existRelacionesReto = $retoDelete->secureDelete("participantes");
            //valido que no haya sido asignado a las personas
            if ($existRelacionesReto)
                return response()->json(['error' => ['mensaje' => ["El reto ha sido asignado"]]], 400);
            //eliminamos el reto
            $retoDelete->delete();
            //Devolvemos la respuesta
            return response()->json([
                'message' => 'Se eliminó el reto correctamente',
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al eliminar el reto"]]], 400);
        }
    }

    /**
     * Asigna los retos
     *
     * @param  \App\Models\Reto  $reto
     * @return \Illuminate\Http\Response
     */
    public function asignReto(Request $request)
    {
        $data = $request->only(
            'reto_id',
            'participantes'
        );
        $validator = Validator::make($data, [
            'reto_id' => 'required|integer|exists:retos,id',
            'participantes' => 'sometimes|array',
            'participantes.*.participante_id' =>  'required|integer'
        ]);

        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }

        try {
            DB::transaction(function () use ($request) {
                //codigo que elimina a los participantes del reto que no se encuentre en el array
                $idFilterNotExits = array_column($request->all(), 'participante_id');
                $idReto = $request->reto_id;
                $partiRetoToDelete = DB::table('reto_participantes')
                    ->where('reto_id', $idReto)
                    ->whereNotIn('participante_id', $idFilterNotExits)->get();
                $partiRelationArrayDelete = $partiRetoToDelete->toArray();

                if (!empty($partiRelationArrayDelete)) {
                    foreach ($partiRelationArrayDelete as $valor) {
                        $idRetoParti = $valor->id;
                        $retoPartiQuery = RetoParticipantes::findOrfail($idRetoParti);
                        $retoPartiQuery->delete();
                    }
                }
                if (!empty($request->participantes)) {
                    foreach ($request->participantes as $valor) {
                        $participanteId = $valor['participante_id'];
                        $idReto = $request->reto_id;
                        $exitsAsignacionReto = DB::table('reto_participantes')
                            ->where('reto_id', '=', $idReto)
                            ->where('participante_id', '=', $participanteId)->exists();
                        if (!$exitsAsignacionReto) {
                            RetoParticipantes::create([
                                "reto_id" => $idReto,
                                "participante_id" => $participanteId
                            ]);
                        }
                    }
                }
            });
            //Respuesta en caso de que todo vaya bien.
            return response()->json([
                'message' => 'Se asignaron correctamente los participantes.'
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al asignar los retos"]]], 400);
        }
    }
}
