<?php

namespace App\Http\Controllers\V1\Reto;

use App\Enums\EstatusEvidencia;
use App\Http\Controllers\Controller;
use App\Models\Evidencia;
use App\Models\EvidenciaRegistro;
use App\Models\File;
use App\Models\Participante;
use App\Models\HistorialRecompensas;
use App\Models\Insignia;
use App\Models\NivelReto;
use App\Models\Producto;
use App\Models\Reto;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Symfony\Component\HttpFoundation\Response;

class RetoParticipanteController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function retosVigentes($idParticipante)
    {
        try {
            $currentDateTime = Carbon::now();
            $dateNow = $currentDateTime->format('Y-m-d');
            $retosQuery = Reto::with(['nivel_retos' => function ($query) use ($dateNow) {
                $query->where('nivel_retos.fecha_inicio', '<=', $dateNow);
                $query->where('nivel_retos.fecha_fin', '>=', $dateNow);
            }])
                ->whereHas('participantes', function ($query) use ($idParticipante) {
                    $query->where('participantes.id', $idParticipante);
                })
                ->get();
            return $retosQuery;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus retos vigentes"]]], 400);
        }
    }


    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getRetosAsignadosParticipante($idParticipante)
    {
        try {

            $retosQuery = Reto::select(
                'retos.id',
                'retos.nombre',
                'retos.descripcion',
                'retos.tipo_reto',
                'retos.file_id'
            )
                ->with(['nivel_retos' => function ($query) {
                    $query->orderBy('meta', 'asc');
                }, 'files'])
                ->whereHas('participantes', function ($query) use ($idParticipante) {
                    $query->where('participantes.id', $idParticipante);
                })
                ->addSelect([
                    DB::raw('(SELECT MIN(nivel_retos.fecha_inicio) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_inicio'),
                    DB::raw('(SELECT MIN(nivel_retos.fecha_fin) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_fin'),
                    DB::raw('(SELECT MAX(nivel_retos.meta) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as meta_total')
                ])
                ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.cantidad) FROM evidencia_retos
                LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
                LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
                WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
                AND evidencias.participante_id = ?) as cantidad_total_acumulada'), [$idParticipante])
                ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.monto) FROM evidencia_retos
                LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
                LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
                WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
                AND evidencias.participante_id = ?) as monto_total_acumulada'), [$idParticipante])
                ->groupBy('retos.id', 'retos.nombre', 'retos.descripcion', 'retos.tipo_reto', 'retos.file_id')
                ->get()
                ->map(function ($reto) {
                    $avance = 0;
                    $meta_total = $reto->meta_total;
                    if ($reto->tipo_reto == 'cantidad') {
                        $cantidadTotalAcumulada = ($reto->cantidad_total_acumulada) ? $reto->cantidad_total_acumulada : 0;
                        $avance = $cantidadTotalAcumulada / $meta_total * 100;
                    } else {
                        $floatMontoTotalAcumulado = ($reto->monto_total_acumulada) ? floatval($reto->monto_total_acumulada) : 0;
                        $avance = $floatMontoTotalAcumulado  / $meta_total * 100;
                    }
                    $reto->avance = ($avance > 100) ? 100 : $avance;
                    return $reto;
                });

            return $retosQuery;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar mis retos"]]], 400);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getRetoDetalleParticipante($idParticipante, $idReto)
    {
        try {
            $reto = Reto::select(
                'retos.id',
                'retos.nombre',
                'retos.descripcion',
                'retos.tipo_reto',
            )
                ->with(['nivel_retos' => function ($query) {
                    $query->orderBy('meta', 'asc');
                }])
                ->with(['nivel_retos.insignias.files', 'nivel_retos.insignias.recompensa.files'])
                ->where('retos.id', $idReto)
                ->whereHas('participantes', function ($query) use ($idParticipante) {
                    $query->where('participantes.id', $idParticipante);
                })
                ->addSelect([
                    DB::raw('(SELECT MIN(nivel_retos.fecha_inicio) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_inicio'),
                    DB::raw('(SELECT MIN(nivel_retos.fecha_fin) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as fecha_fin'),
                    DB::raw('(SELECT MAX(nivel_retos.meta) FROM nivel_retos WHERE nivel_retos.reto_id = retos.id) as meta_total')
                ])
                ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.cantidad) FROM evidencia_retos
                LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
                LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
                WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
                AND evidencias.participante_id = ?) as cantidad_total_acumulada'), [$idParticipante])
                ->selectRaw(DB::raw('(SELECT SUM(evidencia_registros.monto) FROM evidencia_retos
                LEFT JOIN evidencias ON evidencia_retos.evidencia_id = evidencias.id
                LEFT JOIN evidencia_registros ON evidencias.id = evidencia_registros.evidencia_id
                WHERE evidencia_retos.reto_id = retos.id AND evidencias.estatus = "aprobado"
                AND evidencias.participante_id = ?) as monto_total_acumulada'), [$idParticipante])
                ->selectRaw(DB::raw('(SELECT COUNT(*) FROM evidencias
                WHERE evidencias.estatus = "aprobado"
                AND evidencias.participante_id = ?) as total_ev_apro'), [$idParticipante])
                ->selectRaw(DB::raw('(SELECT COUNT(*) FROM evidencias
                WHERE evidencias.estatus = "evaluar"
                AND evidencias.participante_id = ?) as total_ev_pend'), [$idParticipante])
                ->selectRaw(DB::raw('(SELECT COUNT(*) FROM evidencias
                WHERE evidencias.estatus = "rechazada"
                AND evidencias.participante_id = ?) as total_ev_recha'), [$idParticipante])
                ->groupBy('retos.id', 'retos.nombre', 'retos.descripcion', 'retos.tipo_reto')
                ->first();

            if ($reto) {
                $avance = 0;
                $meta_total = $reto->meta_total;
                if ($reto->tipo_reto == 'cantidad') {
                    $cantidadTotalAcumulada = ($reto->cantidad_total_acumulada) ? $reto->cantidad_total_acumulada : 0;
                    $avance = $cantidadTotalAcumulada / $meta_total * 100;
                } else {
                    $floatMontoTotalAcumulado = ($reto->monto_total_acumulada) ? floatval($reto->monto_total_acumulada) : 0;
                    $avance = $floatMontoTotalAcumulado  / $meta_total * 100;
                }
                $reto->avance = ($avance > 100) ? 100 : $avance;
            }
            return $reto;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar el detalle del reto"]]], 400);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getEvidencias($idParticipante)
    {
        try {
            $evidencias = Evidencia::where('participante_id', $idParticipante)
                ->with('participante', 'files', 'retos', 'registros_evidencias.producto')
                ->take(200)
                ->orderBy('fecha_captura', 'desc')
                ->get();
            return response()->json([
                'evidencias' => $evidencias
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => [$e->getMessage()]]], 400);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getRecompensas($idParticipante)
    {
        try {
            $recompensas = HistorialRecompensas::with([
                'nivel_reto.reto_detail',
                'nivel_reto.insignias.recompensa.files', 'participante'
            ])
            ->whereHas('nivel_reto.insignias.recompensa')
                ->where('participante_id', $idParticipante)
                ->get();
            return $recompensas;
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar las recompensas"]]], 400);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getTotalVentaProductosByParticipante($idParticipante)
    {
        try {
            $totalVentaProducto = Producto::select(
                'productos.id',
                'productos.nombre',
                DB::raw('SUM(evidencia_registros.cantidad) as cantidad_total_acumulada'),
                DB::raw('SUM(evidencia_registros.monto) as monto_total_acumulada')
            )
                ->join('evidencia_registros', 'productos.id', '=', 'evidencia_registros.producto_id')
                ->join('evidencias', 'evidencia_registros.evidencia_id', '=', 'evidencias.id')
                ->where('evidencias.participante_id', $idParticipante)
                ->where('evidencias.estatus', 'aprobado')
                ->groupBy('productos.id', 'productos.nombre')
                ->get();

            return response()->json([
                'data' => $totalVentaProducto,
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus datos de venta."]]], 400);
        }
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getTotalPuntosByParticipante($idParticipante)
    {
        try {
            $currentDateTime = Carbon::now();
            $inicioSemana = $currentDateTime->copy()->startOfWeek();
            $finSemana = $currentDateTime->copy()->endOfWeek();
            $inicioMes = $currentDateTime->copy()->startOfMonth();
            $finMes = $currentDateTime->copy()->endOfMonth();

            $totalPuntos = NivelReto::whereHas('recompensa', function ($query) use ($idParticipante) {
                $query->where('historial_recompensas.participante_id', $idParticipante);
            })
                ->get()->pluck('puntos')->sum();
            $puntoSemana = NivelReto::whereHas('recompensa', function ($query) use ($idParticipante) {
                $query->where('historial_recompensas.participante_id', $idParticipante);
            })
                ->where(function ($query) use ($inicioSemana, $finSemana) {
                    $query->whereBetween('nivel_retos.fecha_inicio', [$inicioSemana, $finSemana])
                        ->OrWhereBetween('nivel_retos.fecha_fin', [$inicioSemana, $finSemana]);
                })->get()->pluck('puntos')->sum();

            $puntosMes = NivelReto::whereHas('recompensa', function ($query) use ($idParticipante) {
                $query->where('historial_recompensas.participante_id', $idParticipante);
            })
                ->where(function ($query) use ($inicioMes, $finMes) {
                    $query->whereBetween('nivel_retos.fecha_inicio', [$inicioMes, $finMes])
                        ->orWhereBetween('nivel_retos.fecha_fin', [$inicioMes, $finMes]);
                })
                ->get()->pluck('puntos')->sum();

            return response()->json([
                'total' => $totalPuntos,
                'puntoSemana' => $puntoSemana,
                'puntosMes' => $puntosMes
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus puntos"]]], 400);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getRankingByTypePlayers($tipo_participante)
    {
        try {
            $listVentaTotalParticipante = Participante::whereHas('usuario.roles', function ($query) use ($tipo_participante) {
                $query->where('name', $tipo_participante);
            })
            ->leftJoin('files', 'participantes.file_id', '=', 'files.id')
            ->join('evidencias', 'participantes.id', '=', 'evidencias.participante_id')
                ->join('evidencia_registros', 'evidencia_registros.evidencia_id', '=', 'evidencias.id')
                ->leftJoin('historial_recompensas', 'historial_recompensas.participante_id', '=', 'participantes.id')
                ->where('evidencias.estatus', 'aprobado')
                ->select(
                    'participantes.id',
                    'participantes.nombre_tienda',
                    'participantes.nombre_completo',
                    'files.path',
                    DB::raw('SUM(evidencia_registros.cantidad) as cantidad_total'),
                    DB::raw('SUM(evidencia_registros.monto) as monto_total'),
                    DB::raw('(SELECT SUM(nivel_retos.puntos) FROM nivel_retos 
                    INNER JOIN historial_recompensas ON nivel_retos.id = historial_recompensas.nivel_id 
                    WHERE historial_recompensas.participante_id = participantes.id) as puntos')
                )
                ->groupBy('participantes.id', 'participantes.nombre_tienda', 'participantes.nombre_completo', 'files.path')
                ->orderBy('cantidad_total', 'desc')
                ->orderBy('monto_total', 'desc')
                ->get();

            return response()->json([
                'message' => 'Se consultaron los datos correctamente',
                'data' => $listVentaTotalParticipante
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => [$e->getMessage()]]], 400);
        }
    }

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function getInsigniasByParticipante($idParticipante)
    {
        try {
            $insignias = Insignia::leftJoin('files', 'insignias.file_id', '=', 'files.id')
                ->whereHas('niveles_retos.recompensa', function ($query) use ($idParticipante) {
                    // $query->where('estatus', 'entregado');
                    $query->where('participante_id', $idParticipante);
                })
                ->select('insignias.nombre', 'files.path')
                ->get();
            return response()->json([
                'data' => $insignias
            ], Response::HTTP_OK);
        } catch (\Exception $e) {
            return response()->json(['error' => ['mensaje' => ["Error al consultar tus insignias"]]], 400);
        }
    }
}
