<?php

namespace App\Http\Controllers\V1\Seguridad;

use App\Helpers\Helper;
use App\Http\Controllers\Controller;
use App\Models\Empresa;
use App\Models\Participante;
use Illuminate\Http\Request;
use App\Models\User;
use Tymon\JWTAuth\Exceptions\JWTException;
use Symfony\Component\HttpFoundation\Response;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Tymon\JWTAuth\Facades\JWTAuth;
use Illuminate\Support\Facades\DB;
use App\Models\Persona;

class AuthController extends Controller
{
    //Función que utilizaremos para registrar al usuario
    public function register(Request $request)
    {
        $data = $request->only('email', 'password', 'nombre');
        //Realizamos las validaciones
        $validator = Validator::make($data, [
            'email' => 'required|email|unique:users',
            'password' => 'required|string|min:6|max:12',
            'nombre' => 'required|max:400|string',
        ]);
        //Devolvemos un error si fallan las validaciones
        if ($validator->fails()) {
            return response()->json($validator->messages(), 400);
        }
        try {
            DB::transaction(function () use ($request) {
                //Creamos el nuevo usuario
                $user = User::create([
                    'email' => $request->email,
                    'name' => $request->nombre,
                    'password' => bcrypt($request->password)
                ]);
                //le asigno el rol con la funcion de la libreria de permisos
                $user->assignRole('administrador');
            });
            return response()->json([
                'message' => 'Se registro con exito el usuario'
            ], Response::HTTP_OK);
        } catch (JWTException $e) {
            return response()->json('Error al registrar el usuario', 401);
        }
        return response()->json('Error al registrar el usuario', 401);
    }

    //se confirma la contraseña del usuario para su acceso
    public function confirmarUsuario(Request $request)
    {
        $data = $request->only('password', 'password_confirmation', 'email');
        //Realizamos las validaciones
        $validator = Validator::make($data, [
            'password' => 'required|confirmed|min:6',
            'email' => 'required|email|exists:users,email',
        ]);
        //Devolvemos un error si fallan las validaciones
        if ($validator->fails()) {
            return response()->json($validator->messages(), 400);
        }
        try {
            $userFind = User::where("email", $request->email)
                ->where("password_confirmed", false)
                ->first();
            if(!$userFind)
                return response()->json('El usuario ya confirmo la contraseña o no existe el usuario', 401);
            //Creamos el nuevo usuario
            $userFind->update([
                'password' => bcrypt($request->password),
                'password_confirmed' => true
            ]);
            return response()->json([
                'message' => 'La contraseña se confirmo con exito.'
            ], Response::HTTP_OK);
        } catch (JWTException $e) {
            return response()->json('Error al confirmar la contraseña', 401);
        }
    }

    //Funcion que utilizaremos para hacer login
    public function authenticate(Request $request)
    {
        //Indicamos que solo queremos recibir email y password de la request
        $credentials = $request->only('email', 'password');
        //Validaciones
        $validator = Validator::make($credentials, [
            'email' => 'required|email',
            'password' => 'required|string|min:6|max:12'
        ]);
        //Devolvemos un error de validación en caso de fallo en las verificaciones
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        //Intentamos hacer login
        try {
            if (!$token = auth()->attempt($credentials)) {
                //Credenciales incorrectas.
                return response()->json([
                    'message' => 'El usuario o contraseña son incorrectos',
                ], 401);
            }

            //se encarga de llevar el contador de session por dia
            Helper::check_session_count(auth()->user());
            Auth::user()->getAllPermissions();
            $participante = Participante::leftJoin('files', 'participantes.file_id', '=', 'files.id')
            ->select('participantes.id', 
            'participantes.nombre_tienda',
            'participantes.nombre_completo',
            'participantes.notificacion',
            'files.path')
            ->where('participantes.user_id', Auth::user()->id)
            ->first();
            //Devolvemos el token
            return response()->json([
                'token' => $token,
                'user' => Auth::user(),
                "perfil" => $participante
            ]);
        } catch (JWTException $e) {
            return response()->json([
                'message' => 'Hubo un error al autenticar.',
            ], 500);
        }
    }
    //Función que utilizaremos para eliminar el token y desconectar al usuario
    public function logout(Request $request)
    {
        //Validamos que se nos envie el token
        $validator = Validator::make($request->only('token'), [
            'token' => 'required|string'
        ]);
        //Si falla la validación
        if ($validator->fails()) {
            return response()->json(['error' => $validator->messages()], 400);
        }
        try {
            //Si el token es valido eliminamos el token desconectando al usuario.
            auth()->setToken($request->token)->logout(true);
            return response()->json([
                'success' => true,
                'message' => 'User disconnected'
            ]);
        } catch (JWTException $exception) {
            return response()->json([
                'message' => 'Error al invalidar el token',
            ], 500);
        }
    }
    //Función que utilizaremos para obtener los datos del usuario y validar si el token a expirado.
    public function getUser(Request $request)
    {
        //Validamos que la request tenga el token
        $this->validate($request, [
            'token' => 'required'
        ]);
        //obtenemos el usuario por el token
        $user = auth()->setToken($request->token)->user();
        //Si no hay usuario es que el token no es valido o que ha expirado
        if (!$user)
            return response()->json([
                'message' => 'Invalid token / token expired',
            ], 401);
        //Devolvemos los datos del usuario si todo va bien. 
        return response()->json(['user' => $user]);
    }
}
